# Generated by Django 3.2.8 on 2022-06-30 23:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0009_trainingrun'),
    ]

    operations = [
        migrations.AddField(
            model_name='trainingrun',
            name='task',
            field=models.CharField(default='', max_length=100),
        ),
    ]



# Generated by Django 3.2 on 2021-09-10 15:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='policysearchtask',
            old_name='customer_model_metrics',
            new_name='presearch_model_val_metrics',
        ),
    ]



"""The Masterful GUI central program module. 

This module defines the Masterful GUI program. The program constructs all the
various pieces that each handles part of the experience and combines them
together to create the full Masterful GUI experience.
"""
from pathlib import Path

from absl import flags

from masterful_gui import server
from masterful_gui import application
from masterful_gui import version

FLAGS = flags.FLAGS

# The host to bind the web server to.
_HOSTNAME = 'localhost'

# Default port used for the server.
_DEFAULT_PORT = 7007

# Directory where static files are collected.
_STATIC_DIR = f"{Path(__file__).parent}/static"


class Visualize(object):
  """The Masterful GUI program class, internally called Visualize."""

  def __init__(self, name: str):
    """Initializes the instance.
    
    Args:
      name: An assigned name for the 
    """
    self._name = name

  def main(self, ignored_argv=("",)):
    """Blocking main function for Masterful GUI.
        
    This method is called by `visualize.backend.main.run_main`, which is
    the standard entrypoint for the Masterful GUI command line program.        
    
    Args:
      ignored_argv: Do not pass. Required for Abseil compatibility.
    """
    application.init_settings_env_variable()
    app = application.create_wsgi_app()
    application.make_migrations()
    application.migrate()

    web_server = server.VisualizeServer(_HOSTNAME, _DEFAULT_PORT, app,
                                        _STATIC_DIR)
    web_server.print_serving_messsage(version.__version__)

    try:
      web_server.serve_forever()
    except KeyboardInterrupt:
      pass

    print(f"\n{self._name} stopped.")


"""Defines functionality for scheduling periodic scans for serialzied protos."""

import threading

from masterful_gui.backend.apps.api import dataset_scanner
from masterful_gui.backend.apps.api import training_run_scanner

# Interval for periodic scanning.
_INTERVAL_SEC = 60.0


def schedule_scan(reschedule: bool, interval: float = _INTERVAL_SEC) -> None:
  """Schedules a scan for serialized protos.

  Args:
    reschedule: Whether to reschedule a new scan upon execution.
    interval: The duration in seconds.    
  """
  kwargs = dict(reschedule=reschedule)
  timer = threading.Timer(interval, _scan, None, kwargs)
  timer.start()
  return


def _scan(reschedule: bool) -> None:
  """Scans for serialized protos.
  
  Args:
    reschedule: Whether to reschedule a scan upon execution.
  """
  try:
    training_run_scanner.scan()
    dataset_scanner.scan()
  except Exception as e:
    print(f'Scanning for serialized protos failed: {str(e)}')

  print("Scanning for serialized protos completed.")

  if reschedule:
    schedule_scan(reschedule)


# Generated by Django 3.2 on 2021-08-16 20:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PolicySearchTask',
            fields=[
                ('policy_name', models.CharField(max_length=250, primary_key=True, serialize=False)),
                ('approach_name', models.CharField(max_length=120)),
                ('customer_model_metrics', models.JSONField(default=dict)),
                ('node_search_tasks', models.JSONField(default=dict)),
            ],
        ),
    ]



"""backend URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from django.views.generic import TemplateView
from rest_framework import routers
from django.contrib.staticfiles.storage import staticfiles_storage
from django.views.generic.base import RedirectView
from django.conf import settings
from django.conf.urls.static import static

from masterful_gui.backend.apps.api import views

router = routers.DefaultRouter()
router.register(r'search', views.PolicySearchTaskView, 'search')
router.register(r'dataset', views.DatasetView, 'dataset')
router.register(r'trainingrun', views.TrainingRunView, 'trainingrun')

urlpatterns = [
    path('admin/', admin.site.urls),
    path('scan/', views.scan_view, name='scan'),
    path('api/', include(router.urls)),
    path('', TemplateView.as_view(template_name="index.html")),
    path('favicon.ico',
         RedirectView.as_view(url=staticfiles_storage.url('img/favicon.ico'))),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: masterful/proto/metadata.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='masterful/proto/metadata.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1emasterful/proto/metadata.proto\"\xc9\x07\n\x0b\x44\x61tasetSpec\x12!\n\x05split\x18\x01 \x01(\x0e\x32\x12.DatasetSpec.Split\x12\r\n\x05title\x18\x02 \x01(\t\x12\x19\n\x11total_cardinality\x18\x03 \x01(\x03\x12\x19\n\x11train_cardinality\x18\x04 \x01(\x03\x12\x17\n\x0fval_cardinality\x18\x05 \x01(\x03\x12\x18\n\x10test_cardinality\x18\x06 \x01(\x03\x12/\n\nlabels_map\x18\x07 \x03(\x0b\x32\x1b.DatasetSpec.LabelsMapEntry\x12J\n\x18total_label_distribution\x18\x08 \x03(\x0b\x32(.DatasetSpec.TotalLabelDistributionEntry\x12J\n\x18train_label_distribution\x18\t \x03(\x0b\x32(.DatasetSpec.TrainLabelDistributionEntry\x12\x46\n\x16val_label_distribution\x18\n \x03(\x0b\x32&.DatasetSpec.ValLabelDistributionEntry\x12H\n\x17test_label_distribution\x18\x0b \x03(\x0b\x32\'.DatasetSpec.TestLabelDistributionEntry\x12\x1e\n\nimage_spec\x18\x0c \x01(\x0b\x32\n.ImageSpec\x12!\n\x04task\x18\r \x01(\x0e\x32\x13.ComputerVisionTask\x12\x13\n\x0bnum_classes\x18\x0e \x01(\x05\x1a\x30\n\x0eLabelsMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a=\n\x1bTotalLabelDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a=\n\x1bTrainLabelDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a;\n\x19ValLabelDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a<\n\x1aTestLabelDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"A\n\x05Split\x12\x11\n\rSPLIT_UNKNOWN\x10\x00\x12\x07\n\x03\x41LL\x10\x01\x12\t\n\x05TRAIN\x10\x02\x12\x07\n\x03VAL\x10\x03\x12\x08\n\x04TEST\x10\x04\"o\n\tImageSpec\x12\x1a\n\x05range\x18\x01 \x01(\x0e\x32\x0b.ImageRange\x12\r\n\x05width\x18\x02 \x01(\x05\x12\x0e\n\x06height\x18\x03 \x01(\x05\x12\x10\n\x08\x63hannels\x18\x04 \x01(\t\x12\x15\n\rchannels_last\x18\x05 \x01(\x08*\xf3\x01\n\x12\x43omputerVisionTask\x12 \n\x1c\x43OMPUTER_VISION_TASK_UNKNOWN\x10\x00\x12\x12\n\x0e\x43LASSIFICATION\x10\x01\x12\x19\n\x15\x42INARY_CLASSIFICATION\x10\x02\x12\x1d\n\x19MULTILABEL_CLASSIFICATION\x10\x03\x12\r\n\tDETECTION\x10\x04\x12\x10\n\x0cLOCALIZATION\x10\x05\x12\x19\n\x15SEMANTIC_SEGMENTATION\x10\x06\x12\x19\n\x15INSTANCE_SEGMENTATION\x10\x07\x12\x16\n\x12KEYPOINT_DETECTION\x10\x08*\x95\x01\n\nImageRange\x12\x17\n\x13IMAGE_RANGE_UNKNOWN\x10\x00\x12\x0c\n\x08ZERO_ONE\x10\x01\x12\x13\n\x0fNEG_ONE_POS_ONE\x10\x02\x12\x0c\n\x08ZERO_255\x10\x03\x12\x16\n\x12IMAGENET_CAFFE_BGR\x10\x04\x12\x12\n\x0eIMAGENET_TORCH\x10\x05\x12\x11\n\rCIFAR10_TORCH\x10\x06\x62\x06proto3')
)

_COMPUTERVISIONTASK = _descriptor.EnumDescriptor(
  name='ComputerVisionTask',
  full_name='ComputerVisionTask',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='COMPUTER_VISION_TASK_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLASSIFICATION', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BINARY_CLASSIFICATION', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MULTILABEL_CLASSIFICATION', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DETECTION', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCALIZATION', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SEMANTIC_SEGMENTATION', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INSTANCE_SEGMENTATION', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KEYPOINT_DETECTION', index=8, number=8,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1120,
  serialized_end=1363,
)
_sym_db.RegisterEnumDescriptor(_COMPUTERVISIONTASK)

ComputerVisionTask = enum_type_wrapper.EnumTypeWrapper(_COMPUTERVISIONTASK)
_IMAGERANGE = _descriptor.EnumDescriptor(
  name='ImageRange',
  full_name='ImageRange',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IMAGE_RANGE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ZERO_ONE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NEG_ONE_POS_ONE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ZERO_255', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IMAGENET_CAFFE_BGR', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IMAGENET_TORCH', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CIFAR10_TORCH', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1366,
  serialized_end=1515,
)
_sym_db.RegisterEnumDescriptor(_IMAGERANGE)

ImageRange = enum_type_wrapper.EnumTypeWrapper(_IMAGERANGE)
COMPUTER_VISION_TASK_UNKNOWN = 0
CLASSIFICATION = 1
BINARY_CLASSIFICATION = 2
MULTILABEL_CLASSIFICATION = 3
DETECTION = 4
LOCALIZATION = 5
SEMANTIC_SEGMENTATION = 6
INSTANCE_SEGMENTATION = 7
KEYPOINT_DETECTION = 8
IMAGE_RANGE_UNKNOWN = 0
ZERO_ONE = 1
NEG_ONE_POS_ONE = 2
ZERO_255 = 3
IMAGENET_CAFFE_BGR = 4
IMAGENET_TORCH = 5
CIFAR10_TORCH = 6


_DATASETSPEC_SPLIT = _descriptor.EnumDescriptor(
  name='Split',
  full_name='DatasetSpec.Split',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SPLIT_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRAIN', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VAL', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEST', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=939,
  serialized_end=1004,
)
_sym_db.RegisterEnumDescriptor(_DATASETSPEC_SPLIT)


_DATASETSPEC_LABELSMAPENTRY = _descriptor.Descriptor(
  name='LabelsMapEntry',
  full_name='DatasetSpec.LabelsMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DatasetSpec.LabelsMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DatasetSpec.LabelsMapEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=640,
  serialized_end=688,
)

_DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY = _descriptor.Descriptor(
  name='TotalLabelDistributionEntry',
  full_name='DatasetSpec.TotalLabelDistributionEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DatasetSpec.TotalLabelDistributionEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DatasetSpec.TotalLabelDistributionEntry.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=690,
  serialized_end=751,
)

_DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY = _descriptor.Descriptor(
  name='TrainLabelDistributionEntry',
  full_name='DatasetSpec.TrainLabelDistributionEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DatasetSpec.TrainLabelDistributionEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DatasetSpec.TrainLabelDistributionEntry.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=753,
  serialized_end=814,
)

_DATASETSPEC_VALLABELDISTRIBUTIONENTRY = _descriptor.Descriptor(
  name='ValLabelDistributionEntry',
  full_name='DatasetSpec.ValLabelDistributionEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DatasetSpec.ValLabelDistributionEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DatasetSpec.ValLabelDistributionEntry.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=816,
  serialized_end=875,
)

_DATASETSPEC_TESTLABELDISTRIBUTIONENTRY = _descriptor.Descriptor(
  name='TestLabelDistributionEntry',
  full_name='DatasetSpec.TestLabelDistributionEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DatasetSpec.TestLabelDistributionEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DatasetSpec.TestLabelDistributionEntry.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=877,
  serialized_end=937,
)

_DATASETSPEC = _descriptor.Descriptor(
  name='DatasetSpec',
  full_name='DatasetSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='split', full_name='DatasetSpec.split', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='title', full_name='DatasetSpec.title', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_cardinality', full_name='DatasetSpec.total_cardinality', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='train_cardinality', full_name='DatasetSpec.train_cardinality', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val_cardinality', full_name='DatasetSpec.val_cardinality', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='test_cardinality', full_name='DatasetSpec.test_cardinality', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels_map', full_name='DatasetSpec.labels_map', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_label_distribution', full_name='DatasetSpec.total_label_distribution', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='train_label_distribution', full_name='DatasetSpec.train_label_distribution', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val_label_distribution', full_name='DatasetSpec.val_label_distribution', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='test_label_distribution', full_name='DatasetSpec.test_label_distribution', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_spec', full_name='DatasetSpec.image_spec', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='task', full_name='DatasetSpec.task', index=12,
      number=13, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_classes', full_name='DatasetSpec.num_classes', index=13,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DATASETSPEC_LABELSMAPENTRY, _DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY, _DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY, _DATASETSPEC_VALLABELDISTRIBUTIONENTRY, _DATASETSPEC_TESTLABELDISTRIBUTIONENTRY, ],
  enum_types=[
    _DATASETSPEC_SPLIT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=1004,
)


_IMAGESPEC = _descriptor.Descriptor(
  name='ImageSpec',
  full_name='ImageSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='range', full_name='ImageSpec.range', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='ImageSpec.width', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='ImageSpec.height', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channels', full_name='ImageSpec.channels', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channels_last', full_name='ImageSpec.channels_last', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1006,
  serialized_end=1117,
)

_DATASETSPEC_LABELSMAPENTRY.containing_type = _DATASETSPEC
_DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY.containing_type = _DATASETSPEC
_DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY.containing_type = _DATASETSPEC
_DATASETSPEC_VALLABELDISTRIBUTIONENTRY.containing_type = _DATASETSPEC
_DATASETSPEC_TESTLABELDISTRIBUTIONENTRY.containing_type = _DATASETSPEC
_DATASETSPEC.fields_by_name['split'].enum_type = _DATASETSPEC_SPLIT
_DATASETSPEC.fields_by_name['labels_map'].message_type = _DATASETSPEC_LABELSMAPENTRY
_DATASETSPEC.fields_by_name['total_label_distribution'].message_type = _DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY
_DATASETSPEC.fields_by_name['train_label_distribution'].message_type = _DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY
_DATASETSPEC.fields_by_name['val_label_distribution'].message_type = _DATASETSPEC_VALLABELDISTRIBUTIONENTRY
_DATASETSPEC.fields_by_name['test_label_distribution'].message_type = _DATASETSPEC_TESTLABELDISTRIBUTIONENTRY
_DATASETSPEC.fields_by_name['image_spec'].message_type = _IMAGESPEC
_DATASETSPEC.fields_by_name['task'].enum_type = _COMPUTERVISIONTASK
_DATASETSPEC_SPLIT.containing_type = _DATASETSPEC
_IMAGESPEC.fields_by_name['range'].enum_type = _IMAGERANGE
DESCRIPTOR.message_types_by_name['DatasetSpec'] = _DATASETSPEC
DESCRIPTOR.message_types_by_name['ImageSpec'] = _IMAGESPEC
DESCRIPTOR.enum_types_by_name['ComputerVisionTask'] = _COMPUTERVISIONTASK
DESCRIPTOR.enum_types_by_name['ImageRange'] = _IMAGERANGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatasetSpec = _reflection.GeneratedProtocolMessageType('DatasetSpec', (_message.Message,), {

  'LabelsMapEntry' : _reflection.GeneratedProtocolMessageType('LabelsMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSPEC_LABELSMAPENTRY,
    '__module__' : 'masterful.proto.metadata_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSpec.LabelsMapEntry)
    })
  ,

  'TotalLabelDistributionEntry' : _reflection.GeneratedProtocolMessageType('TotalLabelDistributionEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY,
    '__module__' : 'masterful.proto.metadata_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSpec.TotalLabelDistributionEntry)
    })
  ,

  'TrainLabelDistributionEntry' : _reflection.GeneratedProtocolMessageType('TrainLabelDistributionEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY,
    '__module__' : 'masterful.proto.metadata_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSpec.TrainLabelDistributionEntry)
    })
  ,

  'ValLabelDistributionEntry' : _reflection.GeneratedProtocolMessageType('ValLabelDistributionEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSPEC_VALLABELDISTRIBUTIONENTRY,
    '__module__' : 'masterful.proto.metadata_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSpec.ValLabelDistributionEntry)
    })
  ,

  'TestLabelDistributionEntry' : _reflection.GeneratedProtocolMessageType('TestLabelDistributionEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSPEC_TESTLABELDISTRIBUTIONENTRY,
    '__module__' : 'masterful.proto.metadata_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSpec.TestLabelDistributionEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETSPEC,
  '__module__' : 'masterful.proto.metadata_pb2'
  # @@protoc_insertion_point(class_scope:DatasetSpec)
  })
_sym_db.RegisterMessage(DatasetSpec)
_sym_db.RegisterMessage(DatasetSpec.LabelsMapEntry)
_sym_db.RegisterMessage(DatasetSpec.TotalLabelDistributionEntry)
_sym_db.RegisterMessage(DatasetSpec.TrainLabelDistributionEntry)
_sym_db.RegisterMessage(DatasetSpec.ValLabelDistributionEntry)
_sym_db.RegisterMessage(DatasetSpec.TestLabelDistributionEntry)

ImageSpec = _reflection.GeneratedProtocolMessageType('ImageSpec', (_message.Message,), {
  'DESCRIPTOR' : _IMAGESPEC,
  '__module__' : 'masterful.proto.metadata_pb2'
  # @@protoc_insertion_point(class_scope:ImageSpec)
  })
_sym_db.RegisterMessage(ImageSpec)


_DATASETSPEC_LABELSMAPENTRY._options = None
_DATASETSPEC_TOTALLABELDISTRIBUTIONENTRY._options = None
_DATASETSPEC_TRAINLABELDISTRIBUTIONENTRY._options = None
_DATASETSPEC_VALLABELDISTRIBUTIONENTRY._options = None
_DATASETSPEC_TESTLABELDISTRIBUTIONENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Generated by Django 3.2 on 2021-09-17 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0003_datasetspec'),
    ]

    operations = [
        migrations.RenameField(
            model_name='datasetspec',
            old_name='payload',
            new_name='image_spec',
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='labels_map',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='num_classes',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='split',
            field=models.CharField(default='SPLIT_UNKNOWN', max_length=50),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='task',
            field=models.CharField(default='COMPUTER_VISION_TASK_UNKNOWN', max_length=250),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='test_cardinality',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='test_label_distribution',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='total_cardinality',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='total_label_distribution',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='train_cardinality',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='train_label_distribution',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='val_cardinality',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='datasetspec',
            name='val_label_distribution',
            field=models.JSONField(default=dict),
        ),
    ]



# Generated by Django 3.2.8 on 2022-07-08 18:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0010_trainingrun_task'),
    ]

    operations = [
        migrations.AddField(
            model_name='trainingrun',
            name='model_graph_image',
            field=models.CharField(default='', max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: masterful/proto/layer.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from masterful_gui.backend.proto import action_pb2 as masterful_dot_proto_dot_action__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='masterful/proto/layer.proto',
    package='',
    syntax='proto3',
    serialized_options=None,
    serialized_pb=_b(
        '\n\x1bmasterful/proto/layer.proto\x1a\x1cmasterful/proto/action.proto\"\xb9\x04\n\x05Layer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1e\n\nlayer_type\x18\x02 \x01(\x0e\x32\n.LayerType\x12$\n\x0fordered_actions\x18\x03 \x03(\x0e\x32\x0b.ActionName\x12\x18\n\x10searched_scalars\x18\x04 \x03(\x02\x12\x16\n\x0eoptimal_scalar\x18\x05 \x01(\x02\x12\x13\n\x0btemperature\x18\x06 \x01(\x02\x12\x33\n\x15logits_or_probability\x18\x07 \x01(\x0e\x32\x14.LogitsOrProbability\x12\x16\n\x0enum_magnitudes\x18\x08 \x01(\x05\x12\x15\n\rnum_quantiles\x18\t \x01(\x05\x12\x18\n\x10k_actions_to_run\x18\n \x01(\x05\x12$\n\rsampling_mode\x18\x0b \x01(\x0e\x32\r.SamplingMode\x12\x32\n\x0f\x61\x63tion_mag_grid\x18\x0c \x03(\x0b\x32\x19.Layer.ActionMagGridEntry\x12\x30\n\x0emax_index_grid\x18\r \x03(\x0b\x32\x18.Layer.MaxIndexGridEntry\x1a\x45\n\x12\x41\x63tionMagGridEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.Int32ListValue:\x02\x38\x01\x1a\x44\n\x11MaxIndexGridEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.Int32ListValue:\x02\x38\x01\" \n\x0eInt32ListValue\x12\x0e\n\x06values\x18\x01 \x03(\x05*\xc9\x01\n\tLayerType\x12\x16\n\x12LAYER_TYPE_UNKNOWN\x10\x00\x12*\n&MASTERFUL_SAMPLE_MAG_BY_QUANTILE_LAYER\x10\x01\x12$\n MASTERFUL_REPLACEMENT_RATE_LAYER\x10\x02\x12\x1d\n\x19MASTERFUL_FIXED_MAG_LAYER\x10\x03\x12\x1f\n\x1bMASTERFUL_TEMPERATURE_LAYER\x10\x04\x12\x12\n\x0eIDENTITY_LAYER\x10\x05*A\n\x0cSamplingMode\x12\x19\n\x15SAMPLING_MODE_UNKNOWN\x10\x00\x12\x0b\n\x07UNIFORM\x10\x01\x12\t\n\x05\x45XACT\x10\x02*U\n\x13LogitsOrProbability\x12!\n\x1dLOGITS_OR_PROBABILITY_UNKNOWN\x10\x00\x12\n\n\x06LOGITS\x10\x01\x12\x0f\n\x0bPROBABILITY\x10\x02\x62\x06proto3'
    ),
    dependencies=[
        masterful_dot_proto_dot_action__pb2.DESCRIPTOR,
    ])

_LAYERTYPE = _descriptor.EnumDescriptor(
    name='LayerType',
    full_name='LayerType',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(name='LAYER_TYPE_UNKNOWN',
                                        index=0,
                                        number=0,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(
            name='MASTERFUL_SAMPLE_MAG_BY_QUANTILE_LAYER',
            index=1,
            number=1,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(name='MASTERFUL_REPLACEMENT_RATE_LAYER',
                                        index=2,
                                        number=2,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='MASTERFUL_FIXED_MAG_LAYER',
                                        index=3,
                                        number=3,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='MASTERFUL_TEMPERATURE_LAYER',
                                        index=4,
                                        number=4,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='IDENTITY_LAYER',
                                        index=5,
                                        number=5,
                                        serialized_options=None,
                                        type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=668,
    serialized_end=869,
)
_sym_db.RegisterEnumDescriptor(_LAYERTYPE)

LayerType = enum_type_wrapper.EnumTypeWrapper(_LAYERTYPE)
_SAMPLINGMODE = _descriptor.EnumDescriptor(
    name='SamplingMode',
    full_name='SamplingMode',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(name='SAMPLING_MODE_UNKNOWN',
                                        index=0,
                                        number=0,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='UNIFORM',
                                        index=1,
                                        number=1,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='EXACT',
                                        index=2,
                                        number=2,
                                        serialized_options=None,
                                        type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=871,
    serialized_end=936,
)
_sym_db.RegisterEnumDescriptor(_SAMPLINGMODE)

SamplingMode = enum_type_wrapper.EnumTypeWrapper(_SAMPLINGMODE)
_LOGITSORPROBABILITY = _descriptor.EnumDescriptor(
    name='LogitsOrProbability',
    full_name='LogitsOrProbability',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(name='LOGITS_OR_PROBABILITY_UNKNOWN',
                                        index=0,
                                        number=0,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='LOGITS',
                                        index=1,
                                        number=1,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='PROBABILITY',
                                        index=2,
                                        number=2,
                                        serialized_options=None,
                                        type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=938,
    serialized_end=1023,
)
_sym_db.RegisterEnumDescriptor(_LOGITSORPROBABILITY)

LogitsOrProbability = enum_type_wrapper.EnumTypeWrapper(_LOGITSORPROBABILITY)
LAYER_TYPE_UNKNOWN = 0
MASTERFUL_SAMPLE_MAG_BY_QUANTILE_LAYER = 1
MASTERFUL_REPLACEMENT_RATE_LAYER = 2
MASTERFUL_FIXED_MAG_LAYER = 3
MASTERFUL_TEMPERATURE_LAYER = 4
IDENTITY_LAYER = 5
SAMPLING_MODE_UNKNOWN = 0
UNIFORM = 1
EXACT = 2
LOGITS_OR_PROBABILITY_UNKNOWN = 0
LOGITS = 1
PROBABILITY = 2

_LAYER_ACTIONMAGGRIDENTRY = _descriptor.Descriptor(
    name='ActionMagGridEntry',
    full_name='Layer.ActionMagGridEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='key',
                                    full_name='Layer.ActionMagGridEntry.key',
                                    index=0,
                                    number=1,
                                    type=5,
                                    cpp_type=1,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='value',
                                    full_name='Layer.ActionMagGridEntry.value',
                                    index=1,
                                    number=2,
                                    type=11,
                                    cpp_type=10,
                                    label=1,
                                    has_default_value=False,
                                    default_value=None,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b('8\001'),
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=492,
    serialized_end=561,
)

_LAYER_MAXINDEXGRIDENTRY = _descriptor.Descriptor(
    name='MaxIndexGridEntry',
    full_name='Layer.MaxIndexGridEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='key',
                                    full_name='Layer.MaxIndexGridEntry.key',
                                    index=0,
                                    number=1,
                                    type=5,
                                    cpp_type=1,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='value',
                                    full_name='Layer.MaxIndexGridEntry.value',
                                    index=1,
                                    number=2,
                                    type=11,
                                    cpp_type=10,
                                    label=1,
                                    has_default_value=False,
                                    default_value=None,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b('8\001'),
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=563,
    serialized_end=631,
)

_LAYER = _descriptor.Descriptor(
    name='Layer',
    full_name='Layer',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='name',
                                    full_name='Layer.name',
                                    index=0,
                                    number=1,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='layer_type',
                                    full_name='Layer.layer_type',
                                    index=1,
                                    number=2,
                                    type=14,
                                    cpp_type=8,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='ordered_actions',
                                    full_name='Layer.ordered_actions',
                                    index=2,
                                    number=3,
                                    type=14,
                                    cpp_type=8,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='searched_scalars',
                                    full_name='Layer.searched_scalars',
                                    index=3,
                                    number=4,
                                    type=2,
                                    cpp_type=6,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='optimal_scalar',
                                    full_name='Layer.optimal_scalar',
                                    index=4,
                                    number=5,
                                    type=2,
                                    cpp_type=6,
                                    label=1,
                                    has_default_value=False,
                                    default_value=float(0),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='temperature',
                                    full_name='Layer.temperature',
                                    index=5,
                                    number=6,
                                    type=2,
                                    cpp_type=6,
                                    label=1,
                                    has_default_value=False,
                                    default_value=float(0),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='logits_or_probability',
                                    full_name='Layer.logits_or_probability',
                                    index=6,
                                    number=7,
                                    type=14,
                                    cpp_type=8,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='num_magnitudes',
                                    full_name='Layer.num_magnitudes',
                                    index=7,
                                    number=8,
                                    type=5,
                                    cpp_type=1,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='num_quantiles',
                                    full_name='Layer.num_quantiles',
                                    index=8,
                                    number=9,
                                    type=5,
                                    cpp_type=1,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='k_actions_to_run',
                                    full_name='Layer.k_actions_to_run',
                                    index=9,
                                    number=10,
                                    type=5,
                                    cpp_type=1,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='sampling_mode',
                                    full_name='Layer.sampling_mode',
                                    index=10,
                                    number=11,
                                    type=14,
                                    cpp_type=8,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='action_mag_grid',
                                    full_name='Layer.action_mag_grid',
                                    index=11,
                                    number=12,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='max_index_grid',
                                    full_name='Layer.max_index_grid',
                                    index=12,
                                    number=13,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[
        _LAYER_ACTIONMAGGRIDENTRY,
        _LAYER_MAXINDEXGRIDENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=62,
    serialized_end=631,
)

_INT32LISTVALUE = _descriptor.Descriptor(
    name='Int32ListValue',
    full_name='Int32ListValue',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='values',
                                    full_name='Int32ListValue.values',
                                    index=0,
                                    number=1,
                                    type=5,
                                    cpp_type=1,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=633,
    serialized_end=665,
)

_LAYER_ACTIONMAGGRIDENTRY.fields_by_name['value'].message_type = _INT32LISTVALUE
_LAYER_ACTIONMAGGRIDENTRY.containing_type = _LAYER
_LAYER_MAXINDEXGRIDENTRY.fields_by_name['value'].message_type = _INT32LISTVALUE
_LAYER_MAXINDEXGRIDENTRY.containing_type = _LAYER
_LAYER.fields_by_name['layer_type'].enum_type = _LAYERTYPE
_LAYER.fields_by_name[
    'ordered_actions'].enum_type = masterful_dot_proto_dot_action__pb2._ACTIONNAME
_LAYER.fields_by_name['logits_or_probability'].enum_type = _LOGITSORPROBABILITY
_LAYER.fields_by_name['sampling_mode'].enum_type = _SAMPLINGMODE
_LAYER.fields_by_name[
    'action_mag_grid'].message_type = _LAYER_ACTIONMAGGRIDENTRY
_LAYER.fields_by_name['max_index_grid'].message_type = _LAYER_MAXINDEXGRIDENTRY
DESCRIPTOR.message_types_by_name['Layer'] = _LAYER
DESCRIPTOR.message_types_by_name['Int32ListValue'] = _INT32LISTVALUE
DESCRIPTOR.enum_types_by_name['LayerType'] = _LAYERTYPE
DESCRIPTOR.enum_types_by_name['SamplingMode'] = _SAMPLINGMODE
DESCRIPTOR.enum_types_by_name['LogitsOrProbability'] = _LOGITSORPROBABILITY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Layer = _reflection.GeneratedProtocolMessageType(
    'Layer',
    (_message.Message,),
    {
        'ActionMagGridEntry':
            _reflection.GeneratedProtocolMessageType(
                'ActionMagGridEntry',
                (_message.Message,),
                {
                    'DESCRIPTOR': _LAYER_ACTIONMAGGRIDENTRY,
                    '__module__': 'masterful.proto.layer_pb2'
                    # @@protoc_insertion_point(class_scope:Layer.ActionMagGridEntry)
                }),
        'MaxIndexGridEntry':
            _reflection.GeneratedProtocolMessageType(
                'MaxIndexGridEntry',
                (_message.Message,),
                {
                    'DESCRIPTOR': _LAYER_MAXINDEXGRIDENTRY,
                    '__module__': 'masterful.proto.layer_pb2'
                    # @@protoc_insertion_point(class_scope:Layer.MaxIndexGridEntry)
                }),
        'DESCRIPTOR':
            _LAYER,
        '__module__':
            'masterful.proto.layer_pb2'
        # @@protoc_insertion_point(class_scope:Layer)
    })
_sym_db.RegisterMessage(Layer)
_sym_db.RegisterMessage(Layer.ActionMagGridEntry)
_sym_db.RegisterMessage(Layer.MaxIndexGridEntry)

Int32ListValue = _reflection.GeneratedProtocolMessageType(
    'Int32ListValue',
    (_message.Message,),
    {
        'DESCRIPTOR': _INT32LISTVALUE,
        '__module__': 'masterful.proto.layer_pb2'
        # @@protoc_insertion_point(class_scope:Int32ListValue)
    })
_sym_db.RegisterMessage(Int32ListValue)

_LAYER_ACTIONMAGGRIDENTRY._options = None
_LAYER_MAXINDEXGRIDENTRY._options = None
# @@protoc_insertion_point(module_scope)



"""Masterful GUI app main module.

This module is the main entry point for the app. Should be kept as simple
as possible, it defines the global flags for the app and runs it.
"""

import sys

from absl import app
from absl import flags

import masterful_gui.program as program

# The default port the app runs on.
_DEFAULT_PORT = 7007

# The name assigned to the server instance.
_SERVER_NAME = "Masterful GUI"

# TODO(ray): Add dynamic support across the app for using a variable port value.
# This includes CORS configurations as well as media storage URL generation.
# For now we are keeping this flag disabled untill we implement this support
# to avoid having the app break if users select a different port than the
# default one.
# flags.DEFINE_integer('port', _DEFAULT_PORT, 'The port for running the server')


def run_main():
  """The main function that runs the app."""
  visualize = program.Visualize(_SERVER_NAME)

  try:
    app.run(visualize.main)
  except Exception as e:
    print(f'Running Masterful GUI failed: {str(e)}')
    sys.exit(1)


if __name__ == '__main__':
  run_main()


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: masterful/proto/params.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='masterful/proto/params.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1cmasterful/proto/params.proto\"\x9b\x03\n\x12\x41rchitectureParams\x12\x0c\n\x04task\x18\x01 \x01(\t\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x1b\n\x13\x65nsemble_multiplier\x18\x03 \x01(\x05\x12\x15\n\rbackbone_only\x18\x04 \x01(\x08\x12 \n\x0binput_shape\x18\x05 \x01(\x0b\x32\x0b.InputShape\x12\x13\n\x0binput_range\x18\x06 \x01(\t\x12\x13\n\x0binput_dtype\x18\x07 \x01(\t\x12\x1b\n\x13input_channels_last\x18\x08 \x01(\x08\x12\x19\n\x11prediction_logits\x18\t \x01(\x08\x12\x18\n\x10prediction_dtype\x18\n \x01(\t\x12\x1c\n\x14prediction_structure\x18\x0b \x01(\t\x12\x18\n\x10prediction_shape\x18\x0c \x01(\t\x12\x18\n\x10total_parameters\x18\r \x01(\x03\x12\x1c\n\x14trainable_parameters\x18\x0e \x01(\x03\x12 \n\x18non_trainable_parameters\x18\x0f \x01(\x03\"=\n\nInputShape\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\x12\x10\n\x08\x63hannels\x18\x03 \x01(\x05\"\xcb\x02\n\x12OptimizationParams\x12\x12\n\nbatch_size\x18\x01 \x01(\x05\x12\x16\n\x0e\x64rop_remainder\x18\x02 \x01(\x08\x12\x0e\n\x06\x65pochs\x18\x03 \x01(\x05\x12\x15\n\rlearning_rate\x18\x04 \x01(\t\x12\x1e\n\x16learning_rate_schedule\x18\x05 \x01(\t\x12\x1e\n\x16learning_rate_callback\x18\x06 \x01(\t\x12\x1c\n\x14warmup_learning_rate\x18\x07 \x01(\t\x12\x15\n\rwarmup_epochs\x18\x08 \x01(\x05\x12\x11\n\toptimizer\x18\t \x01(\t\x12\x0c\n\x04loss\x18\n \x03(\t\x12\x1f\n\x17\x65\x61rly_stopping_callback\x18\x0b \x01(\t\x12\x0f\n\x07metrics\x18\x0c \x03(\t\x12\x1a\n\x12readonly_callbacks\x18\r \x03(\t\"\x1f\n\tSslParams\x12\x12\n\nalgorithms\x18\x01 \x03(\t\"\x8a\x02\n\nDataParams\x12\x13\n\x0bnum_classes\x18\x01 \x01(\x05\x12\x0f\n\x07\x63v_task\x18\x02 \x01(\t\x12\x13\n\x0bimage_shape\x18\x03 \x01(\t\x12\x13\n\x0bimage_range\x18\x04 \x01(\t\x12\x13\n\x0bimage_dtype\x18\x05 \x01(\t\x12\x1b\n\x13image_channels_last\x18\x06 \x01(\x08\x12\x13\n\x0blabel_dtype\x18\x07 \x01(\t\x12\x13\n\x0blabel_shape\x18\x08 \x01(\t\x12\x17\n\x0flabel_structure\x18\t \x01(\t\x12\x14\n\x0clabel_sparse\x18\n \x01(\x08\x12!\n\x19label_bounding_box_format\x18\x0b \x01(\tb\x06proto3')
)




_ARCHITECTUREPARAMS = _descriptor.Descriptor(
  name='ArchitectureParams',
  full_name='ArchitectureParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='ArchitectureParams.task', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_classes', full_name='ArchitectureParams.num_classes', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ensemble_multiplier', full_name='ArchitectureParams.ensemble_multiplier', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='backbone_only', full_name='ArchitectureParams.backbone_only', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input_shape', full_name='ArchitectureParams.input_shape', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input_range', full_name='ArchitectureParams.input_range', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input_dtype', full_name='ArchitectureParams.input_dtype', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input_channels_last', full_name='ArchitectureParams.input_channels_last', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prediction_logits', full_name='ArchitectureParams.prediction_logits', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prediction_dtype', full_name='ArchitectureParams.prediction_dtype', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prediction_structure', full_name='ArchitectureParams.prediction_structure', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prediction_shape', full_name='ArchitectureParams.prediction_shape', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_parameters', full_name='ArchitectureParams.total_parameters', index=12,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trainable_parameters', full_name='ArchitectureParams.trainable_parameters', index=13,
      number=14, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='non_trainable_parameters', full_name='ArchitectureParams.non_trainable_parameters', index=14,
      number=15, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=444,
)


_INPUTSHAPE = _descriptor.Descriptor(
  name='InputShape',
  full_name='InputShape',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='width', full_name='InputShape.width', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='InputShape.height', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channels', full_name='InputShape.channels', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=446,
  serialized_end=507,
)


_OPTIMIZATIONPARAMS = _descriptor.Descriptor(
  name='OptimizationParams',
  full_name='OptimizationParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='OptimizationParams.batch_size', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='drop_remainder', full_name='OptimizationParams.drop_remainder', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='epochs', full_name='OptimizationParams.epochs', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='learning_rate', full_name='OptimizationParams.learning_rate', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='learning_rate_schedule', full_name='OptimizationParams.learning_rate_schedule', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='learning_rate_callback', full_name='OptimizationParams.learning_rate_callback', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='warmup_learning_rate', full_name='OptimizationParams.warmup_learning_rate', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='warmup_epochs', full_name='OptimizationParams.warmup_epochs', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='optimizer', full_name='OptimizationParams.optimizer', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loss', full_name='OptimizationParams.loss', index=9,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='early_stopping_callback', full_name='OptimizationParams.early_stopping_callback', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metrics', full_name='OptimizationParams.metrics', index=11,
      number=12, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly_callbacks', full_name='OptimizationParams.readonly_callbacks', index=12,
      number=13, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=510,
  serialized_end=841,
)


_SSLPARAMS = _descriptor.Descriptor(
  name='SslParams',
  full_name='SslParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='algorithms', full_name='SslParams.algorithms', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=843,
  serialized_end=874,
)


_DATAPARAMS = _descriptor.Descriptor(
  name='DataParams',
  full_name='DataParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_classes', full_name='DataParams.num_classes', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cv_task', full_name='DataParams.cv_task', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_shape', full_name='DataParams.image_shape', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_range', full_name='DataParams.image_range', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_dtype', full_name='DataParams.image_dtype', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_channels_last', full_name='DataParams.image_channels_last', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_dtype', full_name='DataParams.label_dtype', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_shape', full_name='DataParams.label_shape', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_structure', full_name='DataParams.label_structure', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_sparse', full_name='DataParams.label_sparse', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_bounding_box_format', full_name='DataParams.label_bounding_box_format', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=877,
  serialized_end=1143,
)

_ARCHITECTUREPARAMS.fields_by_name['input_shape'].message_type = _INPUTSHAPE
DESCRIPTOR.message_types_by_name['ArchitectureParams'] = _ARCHITECTUREPARAMS
DESCRIPTOR.message_types_by_name['InputShape'] = _INPUTSHAPE
DESCRIPTOR.message_types_by_name['OptimizationParams'] = _OPTIMIZATIONPARAMS
DESCRIPTOR.message_types_by_name['SslParams'] = _SSLPARAMS
DESCRIPTOR.message_types_by_name['DataParams'] = _DATAPARAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ArchitectureParams = _reflection.GeneratedProtocolMessageType('ArchitectureParams', (_message.Message,), {
  'DESCRIPTOR' : _ARCHITECTUREPARAMS,
  '__module__' : 'masterful.proto.params_pb2'
  # @@protoc_insertion_point(class_scope:ArchitectureParams)
  })
_sym_db.RegisterMessage(ArchitectureParams)

InputShape = _reflection.GeneratedProtocolMessageType('InputShape', (_message.Message,), {
  'DESCRIPTOR' : _INPUTSHAPE,
  '__module__' : 'masterful.proto.params_pb2'
  # @@protoc_insertion_point(class_scope:InputShape)
  })
_sym_db.RegisterMessage(InputShape)

OptimizationParams = _reflection.GeneratedProtocolMessageType('OptimizationParams', (_message.Message,), {
  'DESCRIPTOR' : _OPTIMIZATIONPARAMS,
  '__module__' : 'masterful.proto.params_pb2'
  # @@protoc_insertion_point(class_scope:OptimizationParams)
  })
_sym_db.RegisterMessage(OptimizationParams)

SslParams = _reflection.GeneratedProtocolMessageType('SslParams', (_message.Message,), {
  'DESCRIPTOR' : _SSLPARAMS,
  '__module__' : 'masterful.proto.params_pb2'
  # @@protoc_insertion_point(class_scope:SslParams)
  })
_sym_db.RegisterMessage(SslParams)

DataParams = _reflection.GeneratedProtocolMessageType('DataParams', (_message.Message,), {
  'DESCRIPTOR' : _DATAPARAMS,
  '__module__' : 'masterful.proto.params_pb2'
  # @@protoc_insertion_point(class_scope:DataParams)
  })
_sym_db.RegisterMessage(DataParams)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: masterful/proto/policy.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from masterful_gui.backend.proto import layer_pb2 as masterful_dot_proto_dot_layer__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='masterful/proto/policy.proto',
    package='',
    syntax='proto3',
    serialized_options=None,
    serialized_pb=_b(
        '\n\x1cmasterful/proto/policy.proto\x1a\x1bmasterful/proto/layer.proto\"B\n\x06Policy\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npolicy_key\x18\x02 \x01(\t\x12\x16\n\x06layers\x18\x03 \x03(\x0b\x32\x06.Layerb\x06proto3'
    ),
    dependencies=[
        masterful_dot_proto_dot_layer__pb2.DESCRIPTOR,
    ])

_POLICY = _descriptor.Descriptor(
    name='Policy',
    full_name='Policy',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='name',
                                    full_name='Policy.name',
                                    index=0,
                                    number=1,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='policy_key',
                                    full_name='Policy.policy_key',
                                    index=1,
                                    number=2,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='layers',
                                    full_name='Policy.layers',
                                    index=2,
                                    number=3,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=61,
    serialized_end=127,
)

_POLICY.fields_by_name[
    'layers'].message_type = masterful_dot_proto_dot_layer__pb2._LAYER
DESCRIPTOR.message_types_by_name['Policy'] = _POLICY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Policy = _reflection.GeneratedProtocolMessageType(
    'Policy',
    (_message.Message,),
    {
        'DESCRIPTOR': _POLICY,
        '__module__': 'masterful.proto.policy_pb2'
        # @@protoc_insertion_point(class_scope:Policy)
    })
_sym_db.RegisterMessage(Policy)

# @@protoc_insertion_point(module_scope)



# Generated by Django 3.2.8 on 2022-08-11 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0012_auto_20220727_1451'),
    ]

    operations = [
        migrations.AddField(
            model_name='trainingrun',
            name='datasets',
            field=models.JSONField(default=dict),
        ),
    ]



# Generated by Django 3.2.8 on 2022-02-14 22:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0005_policysearchtask_engine_version'),
    ]

    operations = [
        migrations.AddField(
            model_name='policysearchtask',
            name='fit_was_captured',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='policysearchtask',
            name='learned_policy_val_metrics',
            field=models.JSONField(default=dict),
        ),
    ]



from rest_framework import serializers

from masterful_gui.backend.apps.api import models


class PolicySearchTaskModelSerializer(serializers.ModelSerializer):

  class Meta:
    model = models.PolicySearchTask
    fields = '__all__'


class DatasetModelSerializer(serializers.ModelSerializer):

  class Meta:
    model = models.Dataset
    fields = '__all__'


class TrainingRunModelSerializer(serializers.ModelSerializer):

  class Meta:
    model = models.TrainingRun
    fields = '__all__'


from django.contrib import admin

from masterful_gui.backend.apps.api import models


class PolicySearchTaskAdmin(admin.ModelAdmin):
  list_display = ['policy_name']


admin.site.register(models.PolicySearchTask, PolicySearchTaskAdmin)



"""
Django settings for backend project in dev environment.

Generated by 'django-admin startproject' using Django 3.2.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

import os

from pathlib import Path
from corsheaders.defaults import default_methods
from corsheaders.defaults import default_headers

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

FRONTEND_STATS_DIR = f"{Path(__file__).resolve().parent.parent.parent}/frontend/stats"

# Used to store various images we display in the GUI that are sourced from
# training runs in Masterful.
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = '/media/'

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-txjpzz7ujwlc#@0ze!8vhru#h6_-=*1trxa9(+66d5@o80pf7_'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'masterful_gui.backend.apps.api.apps.ApiConfig',
    'corsheaders',
    'rest_framework',
    'webpack_loader',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'corsheaders.middleware.CorsMiddleware',
]

ROOT_URLCONF = 'masterful_gui.backend.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, "templates"),],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'masterful_gui.backend.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME':
            'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME':
            'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME':
            'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME':
            'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

STATIC_URL = 'static/'
STATIC_ROOT = BASE_DIR / 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/3.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

# React app will be deployed there.
CORS_ALLOWED_ORIGINS = [
    'http://localhost:3000', 'http://localhost:5000', 'http://localhost:8000',
    'http://localhost:7007,'
]

# CORS configurations
CORS_ALLOW_HEADERS = list(default_headers) + [
    "access-control-allow-headers",
    "access-control-allow-methods",
    "access-control-allow-origin",
]

CORS_ALLOW_METHODS = list(default_methods)

WEBPACK_LOADER = {
    'DEFAULT': {
        'BUNDLE_DIR_NAME':
            'bundles/',
        'STATS_FILE':
            os.path.join(FRONTEND_STATS_DIR, 'webpack-stats.dev.json'),
    }
}

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "assets"),
]


# Generated by Django 3.2 on 2021-09-17 13:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0002_rename_customer_model_metrics_policysearchtask_presearch_model_val_metrics'),
    ]

    operations = [
        migrations.CreateModel(
            name='DatasetSpec',
            fields=[
                ('title', models.CharField(max_length=250, primary_key=True, serialize=False)),
                ('payload', models.JSONField(default=dict)),
            ],
        ),
    ]



# Generated by Django 3.2.8 on 2022-06-30 22:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0008_auto_20220427_1456'),
    ]

    operations = [
        migrations.CreateModel(
            name='TrainingRun',
            fields=[
                ('name', models.CharField(max_length=250, primary_key=True, serialize=False)),
                ('completed_ts', models.DateTimeField()),
                ('model_info', models.JSONField(default=dict)),
                ('policy_search_results', models.JSONField(default=dict)),
                ('approach_name', models.CharField(max_length=120)),
                ('masterful_version', models.CharField(default='', max_length=100)),
            ],
        ),
    ]



"""The Masterful GUI web server module."""

import sys

from werkzeug.serving import run_simple

# TODO(MAS-1169): Set the args based on dev vs. prod mode.
# The current configuration optimizes for prod mode.


class VisualizeServer(object):
  """Handles the creation of the web server that runs the WSGI application.
  
  This class is a wrapper class around the Werkzeug web server functionality.  
  It currently plays the role of abstracting the web serving responsiblity
  to an object, but internally only passes configuration args to the Werkzeug
  API that creates the default web server. As development continues we will
  need to customize ceratin behaviors of the web server, such customiztations
  are going to live here.
  """

  def __init__(self,
               hostname: str,
               port: int,
               app,
               static_files: str,
               use_reloader: bool = False,
               use_debugger: bool = False):
    """Initializes the instance.
    
    Args:
      hostname: The host to bind to. i.e. 'localhost'.
      port: The port the server should use.
      app: The WSGI app the server will execute.
      static_files: The absolute path to the static files dir.
      use_reloader: Whether the server should automatically restart the python
        process if the modules were changed while it's running. Only set True
        for dev, in production this should always be False.
      use_debugger: Enables/disables the werkzeug debugging system.
    """
    self._hostname = hostname
    self._port = port
    self._app = app
    self._static_files = static_files
    self._use_reloader = use_reloader
    self._use_debugger = use_debugger

  def print_serving_messsage(self, version: str):
    """Prints a user-friendly message prior to server start.
    
    This will (should) be called just before `serve_forever`.

    Args:
      version: The current version of the GUI to be displayed in the console
        output.
    
    TODO(ray): Update this after adding a version module or an alternative
    solution.
    
    TODO(ray): Make it pretty add some style to make this standout on the
    terminal.
    """
    sys.stderr.write(
        f"Masterful GUI {version} at http://{self._hostname}:{self._port}/ "
        "(Press CTRL+C to quit)\n\n")
    sys.stderr.flush()

  def serve_forever(self):
    """Runs the server indefinitly."""

    run_simple(
        self._hostname,
        self._port,
        self._app,
        use_reloader=self._use_reloader,
        use_debugger=self._use_debugger,
        static_files={'/static': self._static_files},
    )



import django.http as http
from rest_framework import viewsets

from masterful_gui.backend.apps.api import serializers
from masterful_gui.backend.apps.api import models
from masterful_gui.backend.apps.api import policy_search_task_scanner
from masterful_gui.backend.apps.api import dataset_scanner
from masterful_gui.backend.apps.api import training_run_scanner


class PolicySearchTaskView(viewsets.ReadOnlyModelViewSet):
  serializer_class = serializers.PolicySearchTaskModelSerializer
  queryset = models.PolicySearchTask.objects.all()


class DatasetView(viewsets.ReadOnlyModelViewSet):
  serializer_class = serializers.DatasetModelSerializer
  queryset = models.Dataset.objects.all()


class TrainingRunView(viewsets.ReadOnlyModelViewSet):
  serializer_class = serializers.TrainingRunModelSerializer
  queryset = models.TrainingRun.objects.all().order_by('-completed_ts')


def scan_view(request):
  """This performs scans on all protos supported by visualize."""
  # TODO: improve the experience. Consider reroute or surfacing
  # a template. This is not required it's just for aesthetics.
  try:
    training_run_scanner.scan()
    dataset_scanner.scan()
  except Exception as e:
    print(str(e))
    return http.HttpResponseServerError()

  return http.HttpResponse()



# Generated by Django 3.2.8 on 2022-02-24 00:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0006_auto_20220214_2209'),
    ]

    operations = [
        migrations.AddField(
            model_name='policysearchtask',
            name='task_type',
            field=models.CharField(default='COMPUTER_VISION_TASK_UNKNOWN', max_length=250),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: masterful/proto/search_task.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from masterful_gui.backend.proto import layer_pb2 as masterful_dot_proto_dot_layer__pb2
from masterful_gui.backend.proto import metrics_pb2 as masterful_dot_proto_dot_metrics__pb2
from masterful_gui.backend.proto import metadata_pb2 as masterful_dot_proto_dot_metadata__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='masterful/proto/search_task.proto',
    package='',
    syntax='proto3',
    serialized_options=None,
    serialized_pb=_b(
        '\n!masterful/proto/search_task.proto\x1a\x1bmasterful/proto/layer.proto\x1a\x1dmasterful/proto/metrics.proto\x1a\x1emasterful/proto/metadata.proto\"\xa1\x02\n\x10PolicySearchTask\x12\x13\n\x0bpolicy_name\x18\x01 \x01(\t\x12\x15\n\rapproach_name\x18\x02 \x01(\t\x12*\n\x11node_search_tasks\x18\x03 \x03(\x0b\x32\x0f.NodeSearchTask\x12-\n\x1bpresearch_model_val_metrics\x18\x04 \x01(\x0b\x32\x08.Metrics\x12\x16\n\x0e\x65ngine_version\x18\x05 \x01(\t\x12\x18\n\x10\x66it_was_captured\x18\x06 \x01(\x08\x12,\n\x1alearned_policy_val_metrics\x18\x07 \x01(\x0b\x32\x08.Metrics\x12&\n\ttask_type\x18\x08 \x01(\x0e\x32\x13.ComputerVisionTask\"\xc7\x03\n\x0eNodeSearchTask\x12\x1d\n\x04type\x18\x01 \x01(\x0e\x32\x0f.NodeSearchType\x12\r\n\x05title\x18\x02 \x01(\t\x12\x18\n\x10searched_scalars\x18\x03 \x03(\x02\x12\x16\n\x0eoptimal_scalar\x18\x04 \x01(\x02\x12\'\n\x0bval_metrics\x18\x05 \x03(\x0b\x32\x12.NodeSearchMetrics\x12\x1f\n\x0fsearched_layers\x18\x06 \x03(\x0b\x32\x06.Layer\x12=\n\x10node_scalars_map\x18\x07 \x03(\x0b\x32#.NodeSearchTask.NodeScalarsMapEntry\x12J\n\x17node_optimal_scalar_map\x18\x08 \x03(\x0b\x32).NodeSearchTask.NodeOptimalScalarMapEntry\x1a\x43\n\x13NodeScalarsMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1b\n\x05value\x18\x02 \x01(\x0b\x32\x0c.ScalarsList:\x02\x38\x01\x1a;\n\x19NodeOptimalScalarMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xbd\x01\n\x11NodeSearchMetrics\x12\x17\n\x0fsearched_scalar\x18\x01 \x01(\x02\x12\x19\n\x07metrics\x18\x02 \x01(\x0b\x32\x08.Metrics\x12>\n\x0fnode_scalar_map\x18\x03 \x03(\x0b\x32%.NodeSearchMetrics.NodeScalarMapEntry\x1a\x34\n\x12NodeScalarMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\x1e\n\x0bScalarsList\x12\x0f\n\x07scalars\x18\x01 \x03(\x02*n\n\x0eNodeSearchType\x12\x1c\n\x18NODE_SEARCH_TYPE_UNKNOWN\x10\x00\x12\t\n\x05LAYER\x10\x01\x12\x1b\n\x17SYNTHETIC_DATA_BLENDING\x10\x02\x12\x16\n\x12UNLABELED_DATA_SSL\x10\x03\x62\x06proto3'
    ),
    dependencies=[
        masterful_dot_proto_dot_layer__pb2.DESCRIPTOR,
        masterful_dot_proto_dot_metrics__pb2.DESCRIPTOR,
        masterful_dot_proto_dot_metadata__pb2.DESCRIPTOR,
    ])

_NODESEARCHTYPE = _descriptor.EnumDescriptor(
    name='NodeSearchType',
    full_name='NodeSearchType',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(name='NODE_SEARCH_TYPE_UNKNOWN',
                                        index=0,
                                        number=0,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='LAYER',
                                        index=1,
                                        number=1,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='SYNTHETIC_DATA_BLENDING',
                                        index=2,
                                        number=2,
                                        serialized_options=None,
                                        type=None),
        _descriptor.EnumValueDescriptor(name='UNLABELED_DATA_SSL',
                                        index=3,
                                        number=3,
                                        serialized_options=None,
                                        type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1103,
    serialized_end=1213,
)
_sym_db.RegisterEnumDescriptor(_NODESEARCHTYPE)

NodeSearchType = enum_type_wrapper.EnumTypeWrapper(_NODESEARCHTYPE)
NODE_SEARCH_TYPE_UNKNOWN = 0
LAYER = 1
SYNTHETIC_DATA_BLENDING = 2
UNLABELED_DATA_SSL = 3

_POLICYSEARCHTASK = _descriptor.Descriptor(
    name='PolicySearchTask',
    full_name='PolicySearchTask',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='policy_name',
                                    full_name='PolicySearchTask.policy_name',
                                    index=0,
                                    number=1,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='approach_name',
                                    full_name='PolicySearchTask.approach_name',
                                    index=1,
                                    number=2,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='node_search_tasks',
            full_name='PolicySearchTask.node_search_tasks',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='presearch_model_val_metrics',
            full_name='PolicySearchTask.presearch_model_val_metrics',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='engine_version',
                                    full_name='PolicySearchTask.engine_version',
                                    index=4,
                                    number=5,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='fit_was_captured',
            full_name='PolicySearchTask.fit_was_captured',
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='learned_policy_val_metrics',
            full_name='PolicySearchTask.learned_policy_val_metrics',
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='task_type',
                                    full_name='PolicySearchTask.task_type',
                                    index=7,
                                    number=8,
                                    type=14,
                                    cpp_type=8,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=130,
    serialized_end=419,
)

_NODESEARCHTASK_NODESCALARSMAPENTRY = _descriptor.Descriptor(
    name='NodeScalarsMapEntry',
    full_name='NodeSearchTask.NodeScalarsMapEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='NodeSearchTask.NodeScalarsMapEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='NodeSearchTask.NodeScalarsMapEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b('8\001'),
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=749,
    serialized_end=816,
)

_NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY = _descriptor.Descriptor(
    name='NodeOptimalScalarMapEntry',
    full_name='NodeSearchTask.NodeOptimalScalarMapEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='NodeSearchTask.NodeOptimalScalarMapEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='NodeSearchTask.NodeOptimalScalarMapEntry.value',
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b('8\001'),
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=818,
    serialized_end=877,
)

_NODESEARCHTASK = _descriptor.Descriptor(
    name='NodeSearchTask',
    full_name='NodeSearchTask',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='type',
                                    full_name='NodeSearchTask.type',
                                    index=0,
                                    number=1,
                                    type=14,
                                    cpp_type=8,
                                    label=1,
                                    has_default_value=False,
                                    default_value=0,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='title',
                                    full_name='NodeSearchTask.title',
                                    index=1,
                                    number=2,
                                    type=9,
                                    cpp_type=9,
                                    label=1,
                                    has_default_value=False,
                                    default_value=_b("").decode('utf-8'),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='searched_scalars',
                                    full_name='NodeSearchTask.searched_scalars',
                                    index=2,
                                    number=3,
                                    type=2,
                                    cpp_type=6,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='optimal_scalar',
                                    full_name='NodeSearchTask.optimal_scalar',
                                    index=3,
                                    number=4,
                                    type=2,
                                    cpp_type=6,
                                    label=1,
                                    has_default_value=False,
                                    default_value=float(0),
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='val_metrics',
                                    full_name='NodeSearchTask.val_metrics',
                                    index=4,
                                    number=5,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='searched_layers',
                                    full_name='NodeSearchTask.searched_layers',
                                    index=5,
                                    number=6,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='node_scalars_map',
                                    full_name='NodeSearchTask.node_scalars_map',
                                    index=6,
                                    number=7,
                                    type=11,
                                    cpp_type=10,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='node_optimal_scalar_map',
            full_name='NodeSearchTask.node_optimal_scalar_map',
            index=7,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[
        _NODESEARCHTASK_NODESCALARSMAPENTRY,
        _NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=422,
    serialized_end=877,
)

_NODESEARCHMETRICS_NODESCALARMAPENTRY = _descriptor.Descriptor(
    name='NodeScalarMapEntry',
    full_name='NodeSearchMetrics.NodeScalarMapEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='NodeSearchMetrics.NodeScalarMapEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='NodeSearchMetrics.NodeScalarMapEntry.value',
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b('8\001'),
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1017,
    serialized_end=1069,
)

_NODESEARCHMETRICS = _descriptor.Descriptor(
    name='NodeSearchMetrics',
    full_name='NodeSearchMetrics',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='searched_scalar',
            full_name='NodeSearchMetrics.searched_scalar',
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
        _descriptor.FieldDescriptor(name='metrics',
                                    full_name='NodeSearchMetrics.metrics',
                                    index=1,
                                    number=2,
                                    type=11,
                                    cpp_type=10,
                                    label=1,
                                    has_default_value=False,
                                    default_value=None,
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='node_scalar_map',
            full_name='NodeSearchMetrics.node_scalar_map',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[
        _NODESEARCHMETRICS_NODESCALARMAPENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=880,
    serialized_end=1069,
)

_SCALARSLIST = _descriptor.Descriptor(
    name='ScalarsList',
    full_name='ScalarsList',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(name='scalars',
                                    full_name='ScalarsList.scalars',
                                    index=0,
                                    number=1,
                                    type=2,
                                    cpp_type=6,
                                    label=3,
                                    has_default_value=False,
                                    default_value=[],
                                    message_type=None,
                                    enum_type=None,
                                    containing_type=None,
                                    is_extension=False,
                                    extension_scope=None,
                                    serialized_options=None,
                                    file=DESCRIPTOR),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1071,
    serialized_end=1101,
)

_POLICYSEARCHTASK.fields_by_name[
    'node_search_tasks'].message_type = _NODESEARCHTASK
_POLICYSEARCHTASK.fields_by_name[
    'presearch_model_val_metrics'].message_type = masterful_dot_proto_dot_metrics__pb2._METRICS
_POLICYSEARCHTASK.fields_by_name[
    'learned_policy_val_metrics'].message_type = masterful_dot_proto_dot_metrics__pb2._METRICS
_POLICYSEARCHTASK.fields_by_name[
    'task_type'].enum_type = masterful_dot_proto_dot_metadata__pb2._COMPUTERVISIONTASK
_NODESEARCHTASK_NODESCALARSMAPENTRY.fields_by_name[
    'value'].message_type = _SCALARSLIST
_NODESEARCHTASK_NODESCALARSMAPENTRY.containing_type = _NODESEARCHTASK
_NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY.containing_type = _NODESEARCHTASK
_NODESEARCHTASK.fields_by_name['type'].enum_type = _NODESEARCHTYPE
_NODESEARCHTASK.fields_by_name['val_metrics'].message_type = _NODESEARCHMETRICS
_NODESEARCHTASK.fields_by_name[
    'searched_layers'].message_type = masterful_dot_proto_dot_layer__pb2._LAYER
_NODESEARCHTASK.fields_by_name[
    'node_scalars_map'].message_type = _NODESEARCHTASK_NODESCALARSMAPENTRY
_NODESEARCHTASK.fields_by_name[
    'node_optimal_scalar_map'].message_type = _NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY
_NODESEARCHMETRICS_NODESCALARMAPENTRY.containing_type = _NODESEARCHMETRICS
_NODESEARCHMETRICS.fields_by_name[
    'metrics'].message_type = masterful_dot_proto_dot_metrics__pb2._METRICS
_NODESEARCHMETRICS.fields_by_name[
    'node_scalar_map'].message_type = _NODESEARCHMETRICS_NODESCALARMAPENTRY
DESCRIPTOR.message_types_by_name['PolicySearchTask'] = _POLICYSEARCHTASK
DESCRIPTOR.message_types_by_name['NodeSearchTask'] = _NODESEARCHTASK
DESCRIPTOR.message_types_by_name['NodeSearchMetrics'] = _NODESEARCHMETRICS
DESCRIPTOR.message_types_by_name['ScalarsList'] = _SCALARSLIST
DESCRIPTOR.enum_types_by_name['NodeSearchType'] = _NODESEARCHTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PolicySearchTask = _reflection.GeneratedProtocolMessageType(
    'PolicySearchTask',
    (_message.Message,),
    {
        'DESCRIPTOR': _POLICYSEARCHTASK,
        '__module__': 'masterful.proto.search_task_pb2'
        # @@protoc_insertion_point(class_scope:PolicySearchTask)
    })
_sym_db.RegisterMessage(PolicySearchTask)

NodeSearchTask = _reflection.GeneratedProtocolMessageType(
    'NodeSearchTask',
    (_message.Message,),
    {
        'NodeScalarsMapEntry':
            _reflection.GeneratedProtocolMessageType(
                'NodeScalarsMapEntry',
                (_message.Message,),
                {
                    'DESCRIPTOR': _NODESEARCHTASK_NODESCALARSMAPENTRY,
                    '__module__': 'masterful.proto.search_task_pb2'
                    # @@protoc_insertion_point(class_scope:NodeSearchTask.NodeScalarsMapEntry)
                }),
        'NodeOptimalScalarMapEntry':
            _reflection.GeneratedProtocolMessageType(
                'NodeOptimalScalarMapEntry',
                (_message.Message,),
                {
                    'DESCRIPTOR': _NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY,
                    '__module__': 'masterful.proto.search_task_pb2'
                    # @@protoc_insertion_point(class_scope:NodeSearchTask.NodeOptimalScalarMapEntry)
                }),
        'DESCRIPTOR':
            _NODESEARCHTASK,
        '__module__':
            'masterful.proto.search_task_pb2'
        # @@protoc_insertion_point(class_scope:NodeSearchTask)
    })
_sym_db.RegisterMessage(NodeSearchTask)
_sym_db.RegisterMessage(NodeSearchTask.NodeScalarsMapEntry)
_sym_db.RegisterMessage(NodeSearchTask.NodeOptimalScalarMapEntry)

NodeSearchMetrics = _reflection.GeneratedProtocolMessageType(
    'NodeSearchMetrics',
    (_message.Message,),
    {
        'NodeScalarMapEntry':
            _reflection.GeneratedProtocolMessageType(
                'NodeScalarMapEntry',
                (_message.Message,),
                {
                    'DESCRIPTOR': _NODESEARCHMETRICS_NODESCALARMAPENTRY,
                    '__module__': 'masterful.proto.search_task_pb2'
                    # @@protoc_insertion_point(class_scope:NodeSearchMetrics.NodeScalarMapEntry)
                }),
        'DESCRIPTOR':
            _NODESEARCHMETRICS,
        '__module__':
            'masterful.proto.search_task_pb2'
        # @@protoc_insertion_point(class_scope:NodeSearchMetrics)
    })
_sym_db.RegisterMessage(NodeSearchMetrics)
_sym_db.RegisterMessage(NodeSearchMetrics.NodeScalarMapEntry)

ScalarsList = _reflection.GeneratedProtocolMessageType(
    'ScalarsList',
    (_message.Message,),
    {
        'DESCRIPTOR': _SCALARSLIST,
        '__module__': 'masterful.proto.search_task_pb2'
        # @@protoc_insertion_point(class_scope:ScalarsList)
    })
_sym_db.RegisterMessage(ScalarsList)

_NODESEARCHTASK_NODESCALARSMAPENTRY._options = None
_NODESEARCHTASK_NODEOPTIMALSCALARMAPENTRY._options = None
_NODESEARCHMETRICS_NODESCALARMAPENTRY._options = None
# @@protoc_insertion_point(module_scope)



"""Scanner for serialized TrainingRun protos."""

import os
import pathlib
import datetime
import pytz
import pkg_resources

import google.protobuf.json_format as json_format

from masterful_gui.backend.apps.api import models
from masterful_gui.backend.proto import timestamp_pb2, training_pb2
from masterful_gui.backend.apps.api import constants
from masterful_gui.backend.apps.api import image_utils


def scan():
  """Scans the configured directory for serlized policy protos.
  
  This function deserializes policy protos on disk, converts them
  to Django's owm policy model, and persists them to the database (app model).
  If the database already had a record for that policy, it updates
  that records with the values extracted from the serialized policy. If it
  wasn't, this will persist it to the database.
  """
  _load_sample_binary_classification_run()
  _load_sample_classification_run()

  pathlib.Path(constants.TRAINING_RUN_SCAN_DIR).mkdir(parents=True,
                                                      exist_ok=True)

  if len(os.listdir(constants.TRAINING_RUN_SCAN_DIR)) == 0:
    #TODO(ray): Consider printing something to show in backend logs.
    return

  serialized_runs = []
  for file in os.listdir(constants.TRAINING_RUN_SCAN_DIR):
    if file.split('.')[-1] != constants.TRAINING_RUN_EXT:
      continue
    serialized_runs.append(file)

  if len(serialized_runs) == 0:
    return

  for serialized_run in serialized_runs:
    run = training_pb2.TrainingRun()
    file_name = f'{constants.TRAINING_RUN_SCAN_DIR}/{serialized_run}'
    with open(file_name, "rb") as f:
      run.ParseFromString(f.read())

    run_dict = json_format.MessageToDict(run,
                                         use_integers_for_enums=False,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)

    run_exists = len(models.TrainingRun.objects.filter(pk=run.name)) > 0
    if run_exists:
      existing_run = models.TrainingRun.objects.get(pk=run.name)
      existing_run.model_info = run_dict.get("model_info", {})
      existing_run.policy_search_results = run_dict.get("policy_search_results",
                                                        {})
      existing_run.approach_name = run.approach_name
      existing_run.masterful_version = run.masterful_version
      existing_run.completed_ts = _get_datetime(run.completed_ts)
      existing_run.task = run.task
      if run.model_info.graph_image.graph_was_rendered:
        graph_path = image_utils.save_model_graph_image(
            run.name, run.model_info.graph_image)
        existing_run.model_graph_image = graph_path
      existing_run.optimization_params = run_dict.get("optimization_params", {})
      existing_run.ssl_params = run_dict.get("ssl_params", {})
      existing_run.datasets = run_dict.get("datasets", {})
      existing_run.with_cli = run_dict.get("with_cli", False)
      existing_run.save()

    else:
      graph_path = ''
      if run.model_info.graph_image.graph_was_rendered:
        graph_path = image_utils.save_model_graph_image(
            run.name, run.model_info.graph_image)

      new_run = models.TrainingRun.objects.create(
          name=run.name,
          completed_ts=_get_datetime(run.completed_ts),
          model_info=run_dict.get("model_info", {}),
          policy_search_results=run_dict.get("policy_search_results", {}),
          approach_name=run.approach_name,
          masterful_version=run.masterful_version,
          model_graph_image=graph_path,
          optimization_params=run_dict.get("optimization_params", {}),
          ssl_params=run_dict.get("ssl_params", {}),
          datasets=run_dict.get("datasets", {}),
          with_cli=run_dict.get("with_cli", False),
          task=run.task)
      new_run.save()


def _load_sample_classification_run():
  """Loads the data for a sample classification into the model."""
  loaded_already = len(
      models.TrainingRun.objects.filter(
          pk=constants.CLASSIFICATION_SAMPLE_RUN_NAME)) > 0

  if loaded_already:
    return

  sample_proto = training_pb2.TrainingRun()
  data = pkg_resources.resource_string(
      "masterful_gui", f'data/{constants.CLASSIFICATION_SAMPLE_RUN_FILE_NAME}')
  sample_proto.ParseFromString(data)

  proto_dict = json_format.MessageToDict(sample_proto,
                                         use_integers_for_enums=False,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)
  graph_path = ''
  if sample_proto.model_info.graph_image.graph_was_rendered:
    graph_path = image_utils.save_model_graph_image(
        sample_proto.name, sample_proto.model_info.graph_image)

  model_entry = models.TrainingRun.objects.create(
      name=sample_proto.name,
      completed_ts=_get_datetime(sample_proto.completed_ts),
      model_info=proto_dict.get("model_info", {}),
      policy_search_results=proto_dict.get("policy_search_results", {}),
      approach_name=sample_proto.approach_name,
      masterful_version=sample_proto.masterful_version,
      model_graph_image=graph_path,
      optimization_params=proto_dict.get("optimization_params", {}),
      ssl_params=proto_dict.get("ssl_params", {}),
      task=sample_proto.task,
      datasets=proto_dict.get("datasets", {}),
      with_cli=proto_dict.get("with_cli", False))

  model_entry.save()


def _load_sample_binary_classification_run():
  """Loads the data for a sample binary classification into the model."""
  loaded_already = len(
      models.TrainingRun.objects.filter(
          pk=constants.BINARY_SAMPLE_RUN_NAME)) > 0

  if loaded_already:
    return

  sample_proto = training_pb2.TrainingRun()
  data = pkg_resources.resource_string(
      "masterful_gui", f'data/{constants.BINARY_SAMPLE_RUN_FILE_NAME}')
  sample_proto.ParseFromString(data)

  proto_dict = json_format.MessageToDict(sample_proto,
                                         use_integers_for_enums=False,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)
  graph_path = ''
  if sample_proto.model_info.graph_image.graph_was_rendered:
    graph_path = image_utils.save_model_graph_image(
        sample_proto.name, sample_proto.model_info.graph_image)

  model_entry = models.TrainingRun.objects.create(
      name=sample_proto.name,
      completed_ts=_get_datetime(sample_proto.completed_ts),
      model_info=proto_dict.get("model_info", {}),
      policy_search_results=proto_dict.get("policy_search_results", {}),
      approach_name=sample_proto.approach_name,
      masterful_version=sample_proto.masterful_version,
      model_graph_image=graph_path,
      optimization_params=proto_dict.get("optimization_params", {}),
      ssl_params=proto_dict.get("ssl_params", {}),
      task=sample_proto.task,
      datasets=proto_dict.get("datasets", {}),
      with_cli=proto_dict.get("with_cli", False))

  model_entry.save()


def _get_datetime(timestamp: timestamp_pb2.Timestamp) -> datetime.datetime:
  ts = float(timestamp.seconds + timestamp.nanos / 1e9)
  return datetime.datetime.fromtimestamp(ts, tz=pytz.timezone('UTC'))


"""Define application wide costants for the API server."""

import os

_USER = f'{os.path.expanduser("~")}'

# The location where the scanner should look for serialzed training runs.
TRAINING_RUN_SCAN_DIR = f'{_USER}/.masterful/gui/exports'

# The extension of a serialized TrainingRun proto file. The extension actually
# makes no difference, this is just to facilitate identifying the right files
# to read.
# WARNING: This matches the configuration of gui.exporter module in
# masterful package. You must change them both or you'll break the GUI.
TRAINING_RUN_EXT = "trp"

# The location where the GUI can export artifacts needed for running the app.
# One example is generating PNG images from binary data received from Masterful.
ARTIFACTS_DIR = f'{_USER}/.masterful/gui/data'

MODEL_GRAPH_IMAGE_NAME = 'model_graph_image'

# Dir where model graph image is stored. Nested under MEDIA_ROOT/<run_name>/
MODEL_IMG_DIR = 'model'

# Dir where dataset sample images are stored.
# Nested under MEDIA_ROOT/<run_name>/
DATASET_IMG_DIR = 'dataset'

BINARY_SAMPLE_RUN_NAME = 'Sample run 1'
BINARY_SAMPLE_RUN_FILE_NAME = 'sample_run_bc'
CLASSIFICATION_SAMPLE_RUN_NAME = 'Sample run 2'
CLASSIFICATION_SAMPLE_RUN_FILE_NAME = 'sample_run_c'